/*
** Copyright (C) 2008-2009 SecurixLive   <dev@securixlive.com>
**
** Copyright (C) 2002-2008 Sourcefire, Inc.
** Copyright (C) 1998-2002 Martin Roesch <roesch@sourcefire.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License Version 2 as
** published by the Free Software Foundation.  You may not use, modify or
** distribute this program under any other version of the GNU General
** Public License.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
** Description:
**   A dedicated snort unified2 log file parser
**
** Author(s):
**   firnsy <firnsy@securixlive.com>
**   SecurixLive.com Team <dev@securixlive.com>
**
** Comments:
**   Foundation is built upon the Snort 2.8.3 codebase (www.snort.org/dl) with
** ideas stolen liberally from:
**     1. the orginal barnyard (A. Baker, M. Roesch)
**
*/

#ifndef __BARNYARD2_H__
#define __BARNYARD2_H__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <pcap.h>
#include <stdio.h>

#include <sys/time.h>
#include <sys/stat.h>

#include "sf_types.h"
#include "spooler.h"

/*  I N C L U D E S  **********************************************************/

/*  D E F I N E S  ************************************************************/
#define PROGRAM_NAME	"Barnyard"
#define VER_MAJOR		"2"
#define VER_MINOR		"1"
#define VER_REVISION	"6"
#define VER_BUILD		"163"

#define STD_BUF  1024

#define MAX_PIDFILE_SUFFIX 11 /* uniqueness extension to PID file, see '-R' */

#ifndef _PATH_VARRUN
extern char _PATH_VARRUN[STD_BUF];
#endif

#define DEFAULT_LOG_DIR            "/var/log/barnyard2"
#define DEFAULT_DAEMON_ALERT_FILE  "barnyard2.alert"

#ifdef ACCESSPERMS
    #define FILEACCESSBITS ACCESSPERMS
#else
    #ifdef  S_IAMB
        #define FILEACCESSBITS S_IAMB
    #else
        #define FILEACCESSBITS 0x1FF
    #endif
#endif    

#define TIMEBUF_SIZE    26

#define DO_IP_CHECKSUMS		0x00000001
#define DO_TCP_CHECKSUMS	0x00000003
#define DO_UDP_CHECKSUMS	0x00000004
#define DO_ICMP_CHECKSUMS	0x00000008

#define LOG_UNIFIED         0x00000001
#define LOG_TCPDUMP         0x00000002
#define LOG_UNIFIED2        0x00000004

#define SIGNAL_SNORT_ROTATE_STATS  28
#define SIGNAL_SNORT_CHILD_READY   29

#define BARNYARD2_SUCCESS		0
#define BARNYARD2_EINVAL		1
#define BARNYARD2_ENOMEM		2
#define BARNYARD2_ENOENT		3
#define BARNYARD2_EOPEN			4
#define BARNYARD2_ETRUNC		5
#define BARNYARD2_ECORRUPT		6
#define BARNYARD2_READ_EOF		32
#define BARNYARD2_READ_PARTIAL	33
#define BARNYARD2_FILE_ERROR	34

/*  D A T A  S T R U C T U R E S  *********************************************/

#define MODE_CONTINUOUS     1
#define MODE_BATCH          2
#define MODE_TEST           3
#define MODE_VERSION        4

extern u_int8_t runMode;
extern int datalink;

typedef struct _Configuration
{
    char *logging_directory;

} Configuration;

typedef struct _runtime_config
{
    Configuration configuration;
} runtime_config;

#ifdef MPLS
#define DEFAULT_MPLS_OVERLAPPING_IP		0
#define MPLS_PAYLOADTYPE_IPV4			1
#define MPLS_PAYLOADTYPE_ETHERNET		2
#define MPLS_PAYLOADTYPE_IPV6			3
#define DEFAULT_MPLS_PAYLOADTYPE		1
#define DEFAULT_LABELCHAIN_LENGTH		-1
#endif

/* GetoptLong Option numbers */
#define PID_PATH                  1
#define ARG_RESTART               10
#define CREATE_PID_FILE           11
#define TREAT_DROP_AS_ALERT       12
#define PROCESS_ALL_EVENTS        13
#define ALERT_BEFORE_PASS         14
#define NOLOCK_PID_FILE           15
#define DISABLE_INLINE_INIT       16
#define NO_LOGGING_TIMESTAMPS     18
#define READMODE_LOOP             19

/* struct to contain the program variables and command line args */
typedef struct _progvars
{
	/* staging - snort specific variables */
	int					checksums_mode;
	char				ignore_ports[0x10000];

    /* general variables */
    char				*config_file;
    char				*config_dir;

	char				*hostname;		/* To allow describing this barnyard2 */
	char				*interface;		/* instance similar to that of the   */
	char				*pcap_cmd;		/* snort process's files its reading */

    char				*class_file;
    char				*sid_msg_file;
    char				*gen_msg_file;
    char				*reference_file;
    char				*log_dir;
    char				*chroot_dir;
    u_int8_t			verbose;
    u_int8_t			localtime;
	u_int32_t			event_log_id;
    
    int					quiet_flag;
    int					verbose_flag;
	int					verbose_bytedump_flag;
	int					show2hdr_flag;
	int					char_data_flag;
	int					data_flag;
	int					obfuscation_flag;

	int					logtosyslog_flag;
	int					test_mode_flag;
	
	int					use_utc;
	int					include_year;
	
    int					line_buffer_flag;
    char				nostamp;

#ifdef SUP_IP6
	sfip_t				homenet;
	sfip_t				obfuscation_net;
#else
	u_long				homenet;
	u_long				netmask;
	u_int32_t			obfuscation_net;
	u_int32_t			obfuscation_mask;
#endif

#ifdef MPLS
    u_int8_t overlapping_IP;
    int mpls_stack_depth;
    u_int8_t mpls_payload_type;
#endif

	/* batch mode options */
    int					batch_mode_flag;
	int					batch_total_files;
	char				**batch_filelist;

    /* continual mode options */
	int					new_records_only_flag;
	Waldo				waldo;
    char				*archive_dir;
    int					daemon_flag;
    int					daemon_restart_flag;
    
    /* runtime parameters */
    char pid_filename[STD_BUF];
    char pid_path[STD_BUF];
    char pidfile_suffix[MAX_PIDFILE_SUFFIX+1]; /* room for a null */
    char create_pid_file;
    char nolock_pid_file;
	int done_processing;
	int exit_signal;
    int restart_flag;
    int print_version;
    int usr_signal;
    int cant_hup_signal;

    /* plugin active flags*/
	int input_plugin_active;
	int output_plugin_active;
} PV;

/* struct to collect packet statistics */
typedef struct _PacketCount
{
    UINT64 total_from_pcap;
    UINT64 total_processed;

    UINT64 s5tcp1;
    UINT64 s5tcp2;
    UINT64 ipv6opts;
    UINT64 eth;
    UINT64 ethdisc;
    UINT64 ipv6disc;
    UINT64 ip6ext;
    UINT64 other;
    UINT64 tcp;
    UINT64 udp;
    UINT64 icmp;
    UINT64 arp;
    UINT64 eapol;
    UINT64 vlan;
    UINT64 nested_vlan;
    UINT64 ipv6;
    UINT64 ipv6_up;
    UINT64 ipv6_upfail;
    UINT64 frag6;
    UINT64 icmp6;
    UINT64 tdisc;
    UINT64 udisc;
    UINT64 tcp6;
    UINT64 udp6;
    UINT64 ipdisc;
    UINT64 icmpdisc;
    UINT64 embdip;
    UINT64 ip;
    UINT64 ipx;
    UINT64 ethloopback;

    UINT64 invalid_checksums;

#ifdef GRE
    UINT64 ip4ip4;
    UINT64 ip4ip6;
    UINT64 ip6ip4;
    UINT64 ip6ip6;

    UINT64 gre;
    UINT64 gre_ip;
    UINT64 gre_eth;
    UINT64 gre_arp;
    UINT64 gre_ipv6;
    UINT64 gre_ipv6ext;
    UINT64 gre_ipx;
    UINT64 gre_loopback;
    UINT64 gre_vlan;
    UINT64 gre_ppp;
#endif

    UINT64 discards;
    UINT64 alert_pkts;
    UINT64 log_pkts;
    UINT64 pass_pkts;

    UINT64 frags;           /* number of frags that have come in */
    UINT64 frag_trackers;   /* number of tracking structures generated */
    UINT64 rebuilt_frags;   /* number of packets rebuilt */
    UINT64 frag_incomp;     /* number of frags cleared due to memory issues */
    UINT64 frag_timeout;    /* number of frags cleared due to timeout */
    UINT64 rebuild_element; /* frags that were element of rebuilt pkt */
    UINT64 frag_mem_faults; /* number of times the memory cap was hit */

    UINT64 tcp_stream_pkts; /* number of packets tcp reassembly touches */
    UINT64 rebuilt_tcp;     /* number of phoney tcp packets generated */
    UINT64 tcp_streams;     /* number of tcp streams created */
    UINT64 rebuilt_segs;    /* number of tcp segments used in rebuilt pkts */
    UINT64 queued_segs;     /* number of tcp segments stored for rebuilt pkts */
    UINT64 str_mem_faults;  /* number of times the stream memory cap was hit */

#ifdef TARGET_BASED
    UINT64 attribute_table_reloads; /* number of times attribute table was reloaded. */
#endif

#ifdef DLT_IEEE802_11
  /* wireless statistics */
    UINT64 wifi_mgmt;
    UINT64 wifi_data;
    UINT64 wifi_control; 
    UINT64 assoc_req;
    UINT64 assoc_resp;
    UINT64 reassoc_req;
    UINT64 reassoc_resp;
    UINT64 probe_req;
    UINT64 probe_resp;
    UINT64 beacon;
    UINT64 atim;
    UINT64 dissassoc;
    UINT64 auth;
    UINT64 deauth;
    UINT64 ps_poll;
    UINT64 rts;
    UINT64 cts;
    UINT64 ack;
    UINT64 cf_end;
    UINT64 cf_end_cf_ack;
    UINT64 data;
    UINT64 data_cf_ack;
    UINT64 data_cf_poll;
    UINT64 data_cf_ack_cf_poll;
    UINT64 cf_ack;
    UINT64 cf_poll;
    UINT64 cf_ack_cf_poll;
#endif

#ifdef GIDS
#ifndef IPFW
    UINT64 iptables;
#else
    UINT64 ipfw;
#endif
#endif

#ifdef MPLS
    UINT64 mpls;    
#endif
} PacketCount;

typedef struct _SnortPacketHeader
{
	struct timeval		ts;
	u_int32_t			caplen;
	u_int32_t			pktlen;
} SnortPacketHeader;


/*  G L O B A L S  ************************************************************/
extern PV pv;                 /* program vars (command line args) */
extern PacketCount pc;		/* packet count informaiton) */
extern char *progname;        /* name of the program (from argv[0]) */
extern char **progargs;
extern char *username;
extern char *groupname;
extern unsigned long userid;
extern unsigned long groupid;
extern struct passwd *pw;
extern struct group *gr;

extern u_long netmasks[33];   /* precalculated netmask array */
extern char *protocol_names[256];
extern u_int snaplen;
extern int thiszone;

/* Barnyard2 run-time configuration struct*/
extern runtime_config barnyard_runtime;

/*  P R O T O T Y P E S  ******************************************************/
int BarnyardMain(int argc, char *argv[]);
int BarnyardSleep(unsigned int);
int ParseCmdLine(int, char**);
void *InterfaceThread(void *);
int SetPktProcessor(void);
void CleanExit(int);
void PcapProcessPacket(char *, struct pcap_pkthdr *, u_char *);
int ShowUsage(char *);
void SigCantHupHandler(int signal);
void print_packet_count();

#endif  /* __BARNYARD2_H__ */
