/*
** Copyright (C) 2002-2009 Sourcefire, Inc.
** Copyright (C) 1998-2002 Martin Roesch <roesch@sourcefire.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License Version 2 as
** published by the Free Software Foundation.  You may not use, modify or
** distribute this program under any other version of the GNU General
** Public License.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* $Id$ */
#ifndef __PLUGBASE_H__
#define __PLUGBASE_H__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/ioctl.h>

#ifdef ENABLE_SSL
    #ifdef Free
/* Free macro in radix.h if defined, will conflict with OpenSSL definition */
        #undef Free
    #endif
#endif

#if defined(FREEBSD)
	#include <sys/socket.h>
#endif

#include <net/route.h>

#ifdef ENABLE_SSL
    #undef Free
#endif

#if defined(SOLARIS) || defined(FREEBSD) || defined(OPENBSD)
    #include <sys/param.h>
#endif

#if defined(FREEBSD) || defined(OPENBSD) || defined(NETBSD) || defined(OSF1)
    #include <sys/mbuf.h>
#endif

#ifndef IFNAMSIZ /* IFNAMSIZ is defined in all platforms I checked.. */
    #include <net/if.h>
#endif

#include "decode.h"

#define SMALLBUFFER 32

#define DT_INPUT_UNIFIED_LOG	0x1	/* input node type log_unified */
#define DT_INPUT_UNIFIED_ALERT	0x2	/* input mode type alert_unified */
#define DT_INPUT_UNIFIED		0x3	/* input mode type unified */
#define DT_INPUT_UNIFIED2		0x4 /* input mode type unified2 */

#define NT_OUTPUT_ALERT   0x1  /* output node type alert */
#define NT_OUTPUT_LOG     0x2  /* output node type log */
#define NT_OUTPUT_SPECIAL 0x4  /* special output node type */

#define DETECTION_KEYWORD 0
#define RESPONSE_KEYWORD 1

enum OptionType {
	OPT_TYPE_ACTION = 0,
	OPT_TYPE_LOGGING,
	OPT_TYPE_DETECTION,
	OPT_TYPE_PROCESSOR,
	OPT_TYPE_MAX
};

/***************************** Input Plugin API  ******************************/
typedef void (*InputInitFunc)(char *);
typedef struct _InputKeywordNode
{
    char *keyword;
    InputInitFunc func;
} InputKeywordNode;

typedef struct _InputFuncNode
{
    void *arg;
	int (*readRecordHeader)(void *);
	int (*readRecord)(void *);
} InputFuncNode;

typedef struct _InputKeywordList
{
    InputKeywordNode	entry;

	int					configured_flag;
	InputFuncNode		funcs;

    struct _InputKeywordList *next;
} InputKeywordList;

void InitInputPlugins();
int ActivateInputPlugin(char *plugin_name, char *plugin_options);
void RegisterInputPlugin(char *, InputInitFunc);
InputKeywordList *GetInputPlugin(char *);
void DumpInputPlugins();
int AddArgToInputList(char *plugin_name, void *arg);

int AddReadRecordHeaderFuncToInputList(char *plugin_name, int (*readRecordHeader)(void *));
int AddReadRecordFuncToInputList(char *plugin_name, int (*readRecord)(void *));

int InputFuncNodeConfigured(InputFuncNode *ifn);
/*************************** End Input Plugin API  ****************************/

/***************************** Output Plugin API  *****************************/
typedef void (*OutputInitFunc)(char *);
typedef struct _OutputKeywordNode
{
    char *keyword;
    char node_type;
    OutputInitFunc func;
} OutputKeywordNode;

typedef struct _OutputKeywordList
{
    OutputKeywordNode entry;
    struct _OutputKeywordList *next;

} OutputKeywordList;

typedef struct _OutputFuncNode
{
    void (*func)(Packet *, void *, uint32_t, void *);
    void *arg;
    struct _OutputFuncNode *next;

} OutputFuncNode;

void InitOutputPlugins();
int ActivateOutputPlugin(char *plugin_name, char *plugin_options);
void RegisterOutputPlugin(char *, int, OutputInitFunc);
OutputKeywordNode *GetOutputPlugin(char *plugin_name);
void DumpOutputPlugins();
void AddFuncToOutputList(void (*func) (Packet *, void *, uint32_t, void *),
        char node_type, void *arg);
void CallOutputPlugins(char node_type, Packet *, void *, uint32_t);
/*************************** End Output Plugin API  ***************************/

typedef struct _PluginSignalFuncNode
{
    void (*func)(int, void*);
    void *arg;
    struct _PluginSignalFuncNode *next;
} PluginSignalFuncNode;

void AddFuncToRestartList(void (*func)(int, void*), void*);
void AddFuncToCleanExitList(void (*func)(int, void*), void*);
void AddFuncToShutdownList(void (*func)(int, void*), void*);
void AddFuncToPostConfigList(void (*func)(int, void *), void *);
PluginSignalFuncNode *AddFuncToSignalList(void (*func)(int, void*), void*, PluginSignalFuncNode *);

void PostConfigInitPlugins();

#define ENCODING_HEX 0
#define ENCODING_BASE64 1
#define ENCODING_ASCII 2
#define DETAIL_FAST  0
#define DETAIL_FULL  1

char *GetUniqueName();
char *GetHostname();
int GetLocalTimezone();

/***********************************************************
 If you use any of the functions in this section, you need
 to call free() on the char * that is returned after you are
 done using it. Otherwise, you will have created a memory
 leak.
***********************************************************/
char *GetTimestampByComponent(u_int32_t, u_int32_t, int);
char *GetTimestampByStruct(register const struct timeval *, int);
char *GetCurrentTimestamp();
char *base64(const u_char *, int);
char *ascii(const u_char *, int);
char *hex(const u_char *, int);
char *fasthex(const u_char *, int);
/**********************************************************/

#endif /* __PLUGBASE_H__ */

